package MIDICompose;

public class CombinedControlChange extends MIDIObject implements Channelable, Amountable
{																						// Controllers existing of two controlChanges
	public static final int MIN = 0;
	public static final int MAX = 0x3fff;
	public int ZERO = MIN;																			// depending on controller
	public static final int CENTER = 0x2000;										// for PAN and BALANCE

	public static final int BANK_SELECT = 0;										// all combined controllers
	public static final int MODULATION_WHEEL = 1;
	public static final int MODULATION = 1;
	public static final int BREATH_CONTROLLER = 2;
	public static final int BREATH = 2;
	public static final int FOOT_PEDAL = 4;
	public static final int PORTAMENTO_TIME = 5;
	public static final int DATA_ENTRY = 6;
	public static final int VOLUME = 7;
	public static final int BALANCE = 8;
	public static final int PAN_POSITION = 10;
	public static final int PAN = 10;
	public static final int EXPRESSION = 11;
	public static final int EFFECT_CONTROL_1 = 12;
	public static final int EFFECT1 = 12;
	public static final int EFFECT_CONTROL_2 = 13;
	public static final int EFFECT2 = 13;
	public static final int NON_REGISTERED_PARAMETER = 99;
	public static final int REGISTERED_PARAMETER = 101;

	public static final int REGISTERED_PARAMETER_RESET = 0x3fff;	// for REG_PARAMETER

	public CombinedControlChange(int coarse, int amount) { this(0, 0, coarse, amount); }
	public CombinedControlChange(int deltaTime, int channel, int coarse, int amount) {
		super(deltaTime);
		setController(coarse);
		setChannel(channel);
		setAmount(amount);
		lock();	}

	private int fine(int coarse) {					// returns number of fine Ctrl for coarse Ctrl
		if((coarse >= 0) && (coarse <= 13)) return coarse + 32;
		else if((coarse == 99) || (coarse == 101)) return coarse - 1;
		else {
			System.out.println("ERROR: controller "+coarse+" is not a valid combined controller.");
			System.exit(-1); }
		return -1; }

	public int channel() { return ((ControlChange)elementAt(0)).channel(); }
	public void setChannel(int channel) {
		((ControlChange)elementAt(0)).setChannel(channel);
		((ControlChange)elementAt(1)).setChannel(channel); }

	public int controller() { return ((ControlChange)elementAt(0)).controller(); }
	public void setController(int coarse) {
		if(size() == 0) {
			add(new ControlChange(coarse, 0));
			add(new ControlChange(fine(coarse), 0)); }
		else {
			((ControlChange)elementAt(0)).setController(coarse);
			((ControlChange)elementAt(1)).setController(fine(coarse)); }
		setZero(coarse); }

	private void setZero(int controller) {
		ZERO = MIN;
		switch(controller) {
			case VOLUME: ZERO = MAX; break;
			case PAN:
			case BALANCE:
			case PORTAMENTO_TIME: ZERO = CENTER;
		}}

	public int amount() {
		int amount = (((ControlChange)elementAt(0)).amount() << 7);
		return amount + ((ControlChange)elementAt(1)).amount(); }
	public void setAmount(int amount) {
		if((amount < 0) || (amount > 0x3fff)) {
			System.out.println("ERROR: CombinedControlChange-amount out of range.");
			System.exit(-1); }
		((ControlChange)elementAt(0)).setAmount(amount >> 7);
		((ControlChange)elementAt(1)).setAmount(amount & 0x7f); }

	public int minAmount() { return MIN; }
	public int maxAmount() { return MAX; }
	public int zeroAmount() { return ZERO; }

	public MIDIObject copy() {
		return new CombinedControlChange(deltaTime(), channel(), controller(), amount()); }
}