package MIDICompose;

import java.io.*;

abstract class MIDIEvent extends MIDIObject		// basis-class for MIDI-Events (NoteOn...)
{
	private int absTime;												// used for sorting in MIDIFile-print-list
	private int writeTime;											// deltaTime written to the file (as VLC)

	MIDIEvent(int deltaTime, int status) {
		super(deltaTime); setStatus(status); }		// status byte is added to list

	int absTime() { return absTime; }
	void setAbsTime(int absTime) { this.absTime = absTime; }
	int writeTime() { return writeTime; }
	void setWriteTime(int writeTime) { this.writeTime = writeTime; }

	private void setStatus(int status) {
		if((status > 0xff) || (status < 0x80)) {
			System.out.println("ERROR: no valid status-byte."); System.exit(-1); }
		if(size() == 0) add(new MIDIByte(status));
		else ((MIDIByte)elementAt(0)).setValue(status); }

	protected int dataValueAt(int index) { 				// dataByte at index (can't read status!)
		if(index >= 0) index++;
		return ((MIDIByte)elementAt(index)).value(); }
	protected void setDataValueAt(int value,int index) {	// sets or creates value at index
		if((value > 0x7f) || (value < 0)) {
			System.out.println("ERROR: data-value out of range."); System.exit(-1); }
		if(index >= 0) index++;
		if(size() == index) add(new MIDIByte(value));
		else if(size() > index) ((MIDIByte)elementAt(index)).setValue(value);
		else { System.out.println("ERROR: data-index out of range."); System.exit(-1); }}

	int printSize() {				// returnes Bytes needed in MIDI-file
		int bytes = 0;
		if(writeTime < 0x80) bytes++;
		else if(writeTime < 0x4000) bytes += 2;
		else if(writeTime < 0x200000) bytes += 3;
		else bytes += 4;
		return bytes + size(); }

	void printVarLen(FileOutputStream fpp, int value) throws IOException {
		if(value > 0xfffffff) value = 0xfffffff;
		byte[] byteBuffer = new byte[4];
		int result = value; int count = 0;
		while(result >= 128) {
			byteBuffer[count++] = (byte)((result & 0x7f) | 0x80); result >>= 7; }
		byteBuffer[count++] = (byte)((result & 0x7f) | 0x80); byteBuffer[0] &= 0x7f;
		for(int i = count; i>0; i--) fpp.write(byteBuffer[i-1]); }	// prints VLC

	void print(FileOutputStream fpp) throws IOException {
		printVarLen(fpp, writeTime);
		for(int i = 0; i < size(); i++) fpp.write(((MIDIByte)elementAt(i)).value()); }

	abstract public MIDIObject copy();
}

