package MIDICompose;

import java.io.*;
import java.util.*;

public class MIDIFile extends MIDIContainer		// the MIDIFile with valid print()
{
	private Vector eventList;										// vector for storing all MIDIEvents

	public MIDIFile() { this(120.0, 4, 4); }
	public MIDIFile(double tempo, int numerator, int denominator) {
		this(tempo, numerator, denominator, "Created with MIDICompose (c)2000 Michael Bliem"); }
	public MIDIFile(double tempo, int numerator, int denominator, String copyright) {
		super(0, ABSOLUTE);
		setCopyright(copyright);
		setTempo(tempo);
		setTimeSignature(numerator, denominator); }
	protected MIDIFile(MIDIFile object) { super(object); }

	public String copyright() { return ((TextMetaEvent)elementAt(0)).text(); }
	public void setCopyright(String text) {
		if(size() == 0) add(new TextMetaEvent(0, TextMetaEvent.COPYRIGHT, text));
		else if(size() > 0) ((TextMetaEvent)elementAt(0)).setText(text); }

	public double tempo() { return ((Tempo)elementAt(1)).tempo(); }
	public void setTempo(double tempo) {
		if(size() == 1) add(new Tempo(tempo));
		else if(size() > 1) ((Tempo)elementAt(1)).setTempo(tempo); }

	public int numerator() { return ((TimeSignature)elementAt(2)).numerator(); }
	public void setNumerator(int numerator) {
		if(size() == 2) add(new TimeSignature(numerator, 4));
		else if(size() > 2) ((TimeSignature)elementAt(2)).setNumerator(numerator); }

	public int denominator() { return ((TimeSignature)elementAt(2)).denominator(); }
	public void setDenominator(int denominator) {
		if(size() == 2) add(new TimeSignature(4, denominator));
		else if(size() > 2) ((TimeSignature)elementAt(2)).setDenominator(denominator); }

	public void setTimeSignature(int numerator, int denominator) {
		setNumerator(numerator); setDenominator(denominator); }


// ***************************** Subroutines for print() *********************************


	private void findMIDIEvent(MIDIObject object, int absTime) {	// finds all MIDIEvent
		absTime += object.deltaTime();											// calculates absTime
		for(int i = 0; i < object.size(); i++) {
			if(object.elementAt(i) instanceof MIDIEvent) {		// if a MIDIEvent
				MIDIEvent e = (MIDIEvent)((MIDIEvent)object.elementAt(i)).copy();
				e.setAbsTime(absTime + e.deltaTime());					// sets the absolute time
				addToList(e);	}																	// and adds them to list
			else findMIDIEvent(object.elementAt(i),absTime);	// rekursive
		}}

	private void addToList(MIDIEvent event) {		// MIDIEvents are sorted by absolut time
		boolean done = false;
		for(int i = 0; i < eventList.size(); i++) {
			MIDIEvent e = (MIDIEvent)(eventList.elementAt(i));
			if(event.absTime() < e.absTime()) {
				eventList.insertElementAt(event,i);
				done = true; }
			if(done) break; }
		if(!done) eventList.addElement(event); }

	private int checkList() {										// sets correct deltaTime and returns size
		int bytes = 0;
		int absTime = 0;
		for(int i = 0; i < eventList.size(); i++) {
			MIDIEvent e = (MIDIEvent)(eventList.elementAt(i));
			e.setWriteTime(e.absTime() - absTime);	// writetime is set
			absTime = e.absTime();									// old absTime is stored
			bytes += e.printSize(); }								// bytes is increased by printSize of event
		return bytes; }

	private void printHeader(FileOutputStream fpp) throws IOException {
		fpp.write(new String("MThd").getBytes());								// MThd
		fpp.write(0); fpp.write(0); fpp.write(0); fpp.write(6);	// 6 Bytes follow
		fpp.write(0); fpp.write(0); fpp.write(0); fpp.write(1);	// format=0 numTracks=1
		fpp.write(MIDIVariables.QUARTER >> 8);
		fpp.write(MIDIVariables.QUARTER & 0xff);								// division=QUARTER
		fpp.write(new String("MTrk").getBytes());								// MTrk
		int size = checkList() + 4;															// TrackSize (+endOfTrack)
		byte[] byteBuffer = new byte[4];
		for(int i=0; i<4; i++) { byteBuffer[3-i] = (byte)(size & 0xff); size >>= 8; }
		fpp.write(byteBuffer); }

	private void printList(FileOutputStream fpp) throws IOException {
		for(int i = 0; i < eventList.size(); i++) {
			MIDIEvent e = (MIDIEvent)(eventList.elementAt(i)); e.print(fpp); }
		fpp.write(0); fpp.write(0xff); fpp.write(0x2f); fpp.write(0); }	// endOfTrack

	public void print(FileOutputStream fpp) throws IOException {	// writes the MIDIFile
		eventList = new Vector();
		findMIDIEvent(this,0);								// all MIDIEvents are stored in a list
		printHeader(fpp);											// MIDIFile-header is written to file
		printList(fpp);	}											// all MIDIEvents are written to file

	public MIDIObject copy() { return new MIDIFile(this); }
}
