package MIDICompose;

public class MasterTuning extends MIDIObject implements Channelable
{
	public static final int CENTS_A440 = 0x2000;
	public static final int SEMITONES_A440 = 0x40;

	public MasterTuning(int cents) { this(0, 0, SEMITONES_A440, cents); }
	public MasterTuning(int deltaTime, int channel, int semitones, int cents) {
		this(deltaTime, channel, semitones, cents,
			CombinedControlChange.DATA_ENTRY, CombinedControlChange.DATA_ENTRY + 32); }
	public MasterTuning(int deltaTime, int channel, int semitones, int cents, int coarse, int fine) {
		super(deltaTime);
		add(new CombinedControlChange(0, channel, CombinedControlChange.REGISTERED_PARAMETER, 2));
		add(new ControlChange(0, channel, coarse, semitones));
		add(new CombinedControlChange(0, channel, CombinedControlChange.REGISTERED_PARAMETER, 1));
		add(new ControlChange(0, channel, coarse, 0));
		add(new ControlChange(0, channel, fine, 0));
		setCents(cents);
		lock(); }

	public int channel() { return ((Channelable)elementAt(1)).channel(); }
	public void setChannel(int channel) {
		for(int i = 0; i < size(); i++) ((Channelable)elementAt(i)).setChannel(channel); }

	public int cents() {
		int cents = ((Amountable)elementAt(3)).amount() << 7;
		return cents + ((Amountable)elementAt(4)).amount(); }
	public void setCents(int cents) {
		if((cents < 0) || (cents > 0x3fff)) {
			System.out.println("ERROR: MasterTuning-cents out of range."); System.exit(-1); }
		((Amountable)elementAt(3)).setAmount(cents >> 7);
		((Amountable)elementAt(4)).setAmount(cents & 0x7f); }

	public int semitones() { return ((Amountable)elementAt(1)).amount(); }
	public void setSemitones(int semitones) {
		((Amountable)elementAt(1)).setAmount(semitones); }

	public int coarse() { return ((ControlChange)elementAt(1)).controller(); }
	public void setCoarse(int controller) {
		((ControlChange)elementAt(1)).setController(controller);
		((ControlChange)elementAt(3)).setController(controller); }

	public int fine() { return ((ControlChange)elementAt(4)).controller(); }
	public void setFine(int controller) {
		((ControlChange)elementAt(4)).setController(controller); }

	public MIDIObject copy() {
		return new MasterTuning(deltaTime(), channel(), semitones(), cents(), coarse(), fine()); }
}