package MIDICompose;

public class Tempo extends MetaEvent implements Amountable
{
	public static final int MIN = 20;
	public static final int MAX = 300;
	public static final int ZERO = 120;

	public static final double SLOWLY = 70.0;
	public static final double MODERATELY = 120.0;
	public static final double FAST = 180.0;

	public Tempo(double tempo) { this(0, tempo); }
	public Tempo(int deltaTime, double tempo) {
		super(deltaTime, 0x51); setTempo(tempo); lock(); }

	public double tempo() {
		int size = dataValueAt(1) << 8;
		size = (size + dataValueAt(2)) << 8;
		size += dataValueAt(3);
		return (double)60000000.0 / (double)size; }
	public void setTempo(double tempo) {
		if((tempo < 3.6) || (tempo > 60000000.0)) {
			System.out.println("ERROR: Tempo out of range."); System.exit(-1); }
		int size = (int)Math.round((double)60000000.0 / tempo);
		setDataValueAt(size >> 16, 1);
		setDataValueAt((size >> 8) & 0xff, 2);
		setDataValueAt(size & 0xff, 3); }

	public int amount() { return (int)Math.round(tempo()); }
	public void setAmount(int amount) { setTempo((double)amount); }

	public int minAmount() { return MIN; }
	public int maxAmount() { return MAX; }
	public int zeroAmount() { return ZERO; }

	public MIDIObject copy() { return new Tempo(deltaTime(), tempo()); }
}
