package MIDIComposeUtil;

import MIDICompose.*;

public class Chord implements Channelable
{
	public static final int CLUSTER = 0;
	public static final int NARROW = 1;
	public static final int WIDE = 2;

	private Scale scale;			// Chords for this scale
	private int situation;		// CLUSTER, NARROW or WIDE
	private int degree;				// I, II, IV, ...
	private int channel;			// MIDI-channel

	public Chord(Scale scale) { this(scale, 0, NARROW); }
	public Chord(Scale scale, int degree, int situation) {
		this(0, scale, degree, situation); }
	public Chord(int channel, Scale scale, int degree, int situation) {
		setChannel(channel); setScale(scale); setSituation(situation); setDegree(degree); }

	public int channel() { return channel; }
	public void setChannel(int channel) { this.channel = channel; }

	public Scale scale() { return scale; }
	public void setScale(Scale scale) { this.scale = scale; }

	public int situation() { return situation; }
	public void setSituation(int situation) {
		if((situation < 0) || (situation > 2)) {
			System.out.println("ERROR: situation out of range in Chord."); System.exit(-1); }
		this.situation = situation; }

	public int degree() { return degree; }
	public void setDegree(int degree) { this.degree = degree % scale.octave(); }

	public int pitch(int number) { return pitch(number, 4); }
	public int pitch(int number, int octave) {	// the pitchvalues for each note of chord
		int pitch = number*2 + degree;
		switch(situation) {
			case CLUSTER: while(pitch > scale.octave()) pitch -= scale.octave(); break;
			case NARROW: if((degree > 3) && (number != 1)) pitch -= scale.octave(); break;
			case WIDE:
				if(number == 2)
					if(degree < 4) pitch += scale.octave(); else pitch -= scale.octave();
				if(number == 0) pitch -= scale.octave(); break; }
		return scale.pitch(pitch + octave*scale.octave()); }

	public MIDIContainer notes(int length, int degree) {	// Returns a chord of Notes
		return notes(length, degree, 4, 3); }
	public MIDIContainer notes(int length, int degree, int octave, int number) {
		return notes(0, channel(), length, degree, octave, number, MIDIVariables.MEDIUM); }
	public MIDIContainer notes(int deltaTime, int channel, int length, int degree, int octave, int number, int velocity) {
		MIDIContainer notes = new MIDIContainer(deltaTime, MIDIContainer.FREE, channel);
		setDegree(degree);
		for(int i = 0; i < number; i++)
			notes.appendAt(new Note(0, channel, length, pitch(i, octave), velocity, velocity),0);
		return notes; }
}