package MIDIComposeUtil;

public class Scale
{
	public static final int CHROMATIC = 0;

	public static final int IONIC = 1;
	public static final int DORIC = 2;
	public static final int PHRYGIC = 3;
	public static final int LYDIC = 4;
	public static final int MIXOLYDIC = 5;
	public static final int AEOLIC = 6;
	public static final int LOCRIC = 7;

	public static final int MAJOR = 1;
	public static final int MINOR = 6;
	public static final int HARMONIC_MINOR = 8;
	public static final int MELODIC_MINOR = 9;
	public static final int GIPSY_MINOR = 10;

	public static final int MAJOR_PENTATONIC = 11;
	public static final int MINOR_PENTATONIC = 12;
	public static final int JAPANESE_PENTATONIC = 13;
	public static final int INDIAN_PENTATONIC = 14;
	public static final int BLUES = 15;

	public static final int WHOLETONE = 16;
	public static final int SEMITONE_WHOLETONE = 17;
	public static final int WHOLETONE_SEMITONE = 18;

	private static final int[][] scale = {
		{1,2,3,4,5,6,7,8,9,10,11},					// CHROMATIC
		{2,4,5,7,9,11},											// IONIC, MAJOR
		{2,3,5,7,9,10},											// DORIC
		{1,3,5,7,8,10},											// PHRYGIC
		{2,4,6,7,9,11},											// LYDIC
		{2,4,5,7,9,10},											// MIXOLYDIC
		{2,3,5,7,8,10},											// AEOLIC, MINOR
		{1,3,5,6,8,10},											// LOCRIC
		{2,3,5,7,8,11},											// HARMONIC_MINOR
		{2,3,5,7,9,11},											// MELODIC_MINOR
		{2,3,6,7,8,11},											// GIPSY_MINOR
		{2,4,7,9},													// MAJOR_PENTATONIC
		{3,5,7,10},													// MINOR_PENTATONIC
		{1,5,7,10},													// JAPANESE_PENTATONIC
		{4,5,7,10},													// INDIAN_PENTATONIC
		{3,4,5,6,7,10},											// BLUES
		{2,4,6,8,10},												// WHOLETONE
		{1,3,4,6,7,9,10},										// SEMITONE_WHOLETONE
		{2,3,5,6,8,9,11}										// WHOLETONE_SEMITONE
	};

	private int key;		// key note
	private int type;		// type (MINOR, MAJOR, ...)
	private int root;		// root (standard) octave

	public Scale(int key) { this(key, MAJOR); }
	public Scale(int key, int type) { setType(type); setKey(key); }

	public int key() { return key; }
	public void setKey(int key) { this.key = key % 12; }

	public int type() { return type; }
	public void setType(int type) {
		if((type < 0) || (type > 18)) {
			System.out.println("ERROR: type out of range in Scale."); System.exit(-1); }
		this.type = type; }

	public int octave() { return scale[type].length + 1; }	// number of notes in octave

	public int pitch(int degree, int octave) { return pitch(octave*octave() + degree); }
	public int pitch(int pitch) {				// pitch is a int from 0 to <127 dep. on type
		int octave = pitch / octave();
		int key = pitch % octave();
		int midiPitch = 12 * octave + key();
		if(key > 0) midiPitch += scale[type][key-1];
		if((midiPitch < 0) || (midiPitch > 127)) {
			System.out.println("ERROR: pitch out of range in Scale."); System.exit(-1); }
		return midiPitch; }
}